import json
import csv
import sys
from datetime import datetime, timedelta
import time
sys.path.append("../customerService")
from sendSMS import MessageHandler
from CustomerDetail import CustomerDetail
from BankSalesPerson import BankSalesPerson
from CustomerQuery import CustomerQuery
from CustomerProduct import CustomerProduct
import configparser

class Bank:
    def __init__(self):
        self.customerMap = {}
        self.bankSalesPersonMap = {}
        config = configparser.ConfigParser()
        config.read('../resources/config.ini')

        base_url = config['API']['base_url']
        app_key = config['API']['app_key']
        app_secret = config['API']['app_secret']
        self.messageHandler = MessageHandler(base_url, app_key, app_secret)
    
    def loadDetails(self):
        # loading customer list
        with open('../resources/CustomerList.csv', 'rt') as csvfile:
            reader = csv.reader(csvfile, delimiter='|')
            for row in reader:
                products = row[3:]
                id = row[0]
                self.customerMap[id] = CustomerDetail(row[1], id, row[2])
                for product in products:
                    self.customerMap[id].products.append(CustomerProduct(id, product))
        
        # loading customer queries
        with open('../resources/CustomerQuery.csv', 'rt') as csvQueryFile:
            reader = csv.DictReader(csvQueryFile, delimiter='|', fieldnames=("customerID","type","content"))
            for row in reader:
                customerID = row['customerID']
                if customerID in self.customerMap:
                    self.customerMap[customerID].queries.append(CustomerQuery(customerID, row['type'], row['content']))
        
        # loading bank sales person list
        with open('../resources/BankSalesPersonList.csv', 'rt') as csvBankFile:
            reader = csv.DictReader(csvBankFile, delimiter='|', fieldnames=("id","name","department","phone"))
            for row in reader:
                id = row['id']
                self.bankSalesPersonMap[id] = BankSalesPerson(id, row['name'], row['department'], row['phone'])
    
    def sendAdvertisements(self):
        for key, customer in self.customerMap.items():
            self.creditCardAdvertisment(customer)
    
    def updateComplaints(self):
        for key, customer in self.customerMap.items():
            self.updateComplaint(customer)
    
    def creditCardAdvertisment(self, customer):
        if any(item.product == "CREDIT CARD" for item in customer.products) and not any(item.product == "PREMIUM CARD" for item in customer.products):
            SMSContent = "Hi " + customer.customerName + ", We are offering a new Premium Credit card for you. Upgrade your card to Premium Card with lots of Benefits"
            self.messageHandler.sendSMS(SMSContent, customer.mobileNumber)

    def updateComplaint(self, customer):
        SMSContent = ''
        for query in customer.queries:
            if query.queryType == 'COMPLAINT':
                person = self.getBankPerson(query.queryType)
                SMSContent = "Hi " + customer.customerName + " on your complaint " + query.queryContent + " Mr/Ms " + person.bankPersonName + " will contact you Mobile number: " + person.mobileNumber
            elif query.queryType == 'QUERY':
                person = self.getBankPerson(query.queryType)
                SMSContent = "Hi " + customer.customerName + " on your query " + query.queryContent + " Please connect to our sales person Mr/Ms " + person.bankPersonName + " on " + person.mobileNumber + " Or our sales person will connect with you in next 3 working days."
            elif query.queryType == 'CRITICAL':
                person = self.getBankPerson(query.queryType)
                SMSContent = "Hi " + customer.customerName + " on your issue " + query.queryContent + " an appointment with manager is being fixed on " + self.getDateAndTime() + ""
            self.messageHandler.sendSMS(SMSContent, customer.mobileNumber)
    
    def getDateAndTime(self):
        sysDate = datetime.utcnow()
        days = 1
        if sysDate.weekday() > 5:
            days = 3
        new_time = sysDate + timedelta(days=days)
        return new_time.strftime('%a %b %d %Y')

    def getBankPerson(self, type):
        if type == 'COMPLAINT':
            return self.bankSalesPersonMap['3']
        elif type == 'QUERY':
            return self.bankSalesPersonMap['2']
        elif type == 'CRITICAL':
            return self.bankSalesPersonMap['1']